/**
 * Class for a node of a binary search tree.
 */
public class Node {

	/** The data stored in the node. */
	public Comparable data;

	/** The node's parent. */
	public Node parent;

	/** The node's balanceFactor. */
	public int bf;

	/** The node's left child. */
	public Node left;

	/** The node's right child. */
	public Node right;

	/**
	 * Initializes a node with the key and the data and makes other pointers
	 * null. The Balance Factor is initialized to be 0.
	 * 
	 * @param data
	 *            Data to save in the node.
	 */
	public Node(Comparable data) {
		this.data = data;
		this.parent = null;
		this.left = null;
		this.right = null;
		this.bf = 0;
	} //Node

	/**
	 * Performs a left rotation of the subtree rooted at Node a, and returns the
	 * new root. The pointers and balance factors are updated as specified in
	 * the AVL handout.
	 */

	public Node rotateLeft() {
		Node b = this.right;
		Node a = b.parent;

		a.right = b.left;
		if (a.right != null)
			a.right.parent = a;

		b.left = a;
		if (b.left != null)
			a.parent = b;

		if (b.bf == +1) {
			a.bf = 0;
			b.bf = 0;
		} else { //b.bf == 0
			a.bf = +1;
			b.bf = -1;
		}

		return b;

	} //rotateLeft

	/**
	 * Performs a right rotation of the subtree rooted at Node a, and returns
	 * the new root. The pointers and balance factors are updated as specified
	 * in the AVL handout.
	 */

	public Node rotateRight() {
		Node b = this.left;
		Node a = b.parent;

		a.left = b.right;
		if (a.left != null)
			a.left.parent = a;

		b.right = a;
		if (b.right != null)
			a.parent = b;

		if (b.bf == -1) {
			a.bf = 0;
			b.bf = 0;
		} else { //b.bf == 0
			a.bf = -1;
			b.bf = +1;
		}

		return b;

	} //rotateRight

	/**
	 * Performs a double right left rotation of the subtree rooted at Node a,
	 * and returns the new root. The pointers and balance factors are updated as
	 * specified in the AVL handout.
	 */

	public Node rotateRightLeft() {
		Node b = this.right;
		Node a = b.parent;
		Node c = b.left;

		a.right = c.left;
		if (a.right != null)
			a.right.parent = a;

		b.left = c.right;
		if (b.left != null)
			b.left.parent = b;

		//Updates the balance factors differently depending on
		//whether node c had a balance factor of +1 or -1
		if (c.bf == -1) {
			a.bf = 0;
			b.bf = +1;
		} else if (c.bf == 0) {
			a.bf = 0;
			b.bf = 0;
		} else {
			a.bf = -1;
			b.bf = 0;
		}

		c.left = a;
		a.parent = c;

		c.right = b;
		b.parent = c;

		c.bf = 0;

		return c;

	} //rotateRightLeft

	/**
	 * Performs a double left right rotation of the subtree rooted at Node a,
	 * and returns the new root. The pointers and balance factors are updated as
	 * specified in the AVL handout.
	 */
	public Node rotateLeftRight() {
		Node b = this.left;
		Node a = b.parent;
		Node c = b.right;

		a.left = c.right;
		if (a.left != null)
			a.left.parent = a;

		b.right = c.left;
		if (b.right != null)
			b.right.parent = b;

		//Updates the balance factors differently depending on
		//whether node c had a balance factor of +1 or -1
		if (c.bf == -1) {
			b.bf = 0;
			a.bf = +1;
		} else if (c.bf == 0) {
			b.bf = 0;
			a.bf = 0;
		} else {
			b.bf = -1;
			a.bf = 0;
		}

		c.left = b;
		b.parent = c;

		c.right = a;
		a.parent = c;

		c.bf = 0;

		return c;

	} //rotateLeftRight

} //Node class
